//Call dibs on slots for sounds
freeslot("sfx_fstep0", "sfx_kstep1")

//Decide on the player's feet sounds based on their current skin
addHook("PlayerThink", function(player)
	if not player.mo
		return
	end
	
	// Assign feet sounds to each of the default characters
	// feet ["example"] = {
	//							landingsfx = {slow = sfx_ltplop, fast = sfx_hvplop} // fast is optional
	//							walksfx = {sfx_fstep1, sfx_fstep2, sfx_fstep3, ...}
	//							runsfx = {sfx_fstepa, sfx_fstepb, sfx_fstepc, ...}
	//							walktiming = function(x) return x==2 or x==6 end,	// x is the character's animation frame
	//							runtiming = function(x) return x==1 or x==3 end,
	//							airwalk = true // Set if the character's feet don't touch the ground walking
	//							airrun = true // Set if the character's feet don't touch the ground running
	//						}
	
	local feet = {}
	feet["sonic"] = {
						landingsfx = {slow = nil},
						walksfx = {nil},
						runsfx = {nil},
						walktiming = function(x) return x==2 or x==6 end,
						runtiming = function(x) return x==1 or x==3 end,
					}
	feet["tails"] = {
						landingsfx = {slow = nil, fast = nil},
						walksfx = {nil},
						runsfx = {sfx_putput},
						walktiming = function(x) return x==3 or x==7 end,
						runtiming = function() return leveltime % 10 == 0 or leveltime % 10 == 1 end,
						airrun = true
					}
	feet["knuckles"] = {
						landingsfx = {slow = sfx_s3k4c},
						walksfx = {sfx_kstep1},
						runsfx = {sfx_kstep1},
						walktiming = function(x) return x==2 or x==6 end,
						runtiming = function() return x==1 or x==3 end,
					}
	feet["amy"] = {
						landingsfx = {slow = nil},
						walksfx = {nil},
						runsfx = {nil},
						walktiming = function(x) return x==3 or x==7 end,
						runtiming = function(x) return x==2 or x==6 end,
					}
	feet["fang"] = {
						landingsfx = {slow = nil},
						walksfx = {nil},
						runsfx = {nil},
						walktiming = function(x) return x==3 or x==7 end,
						runtiming = function(x) return x==1 or x==4 end,
					}
	feet["metalsonic"] = {
						landingsfx = {slow = nil},
						walksfx = {nil},
						runsfx = {nil},
						walktiming = function(x) return x==2 or x==6 end,
						runtiming = function(x) return x==1 or x==3 end,
						airwalk = true,
						airrun = true
					}

	// Pick feet sounds for the player based on their skin, default to Sonic's
	player.feet = feet[player.mo.skin] or feet["sonic"]
end)

// Footstep effect function
local function footSteps(player)
	// Only play a sound at the start of an animation frame
	if player.mo.frame == player.prevframe
		return
	end
	
	local stepsound = nil
	local stepvolume = 255
	local minvolume = 102
	
	if player.mo.state == S_PLAY_WALK
	and (P_IsObjectOnGround(player.mo) or player.feet["airwalk"])
		// Check if this is an appropriate frame to play footstep sounds on
		if (player.feet["walktiming"](player.mo.frame))
			// Randomly pick one of available walking footstep sounds
			stepsound = player.feet["walksfx"][P_RandomRange(1, table.getn(player.feet["walksfx"]))]
		else
			return
		end
		// Scale footstep volume to how close player is to running animation
		stepvolume = 255 * player.speed / player.runspeed
	elseif player.mo.state == S_PLAY_RUN
	and (P_IsObjectOnGround(player.mo) or player.feet["airrun"])
		// Check if this is an appropriate frame to play footstep sounds on
		if (player.feet["runtiming"](player.mo.frame))
			stepsound = player.feet["runsfx"][P_RandomRange(1, table.getn(player.feet["runsfx"]))]
		else
			return
		end
	// Not walking or running, no footsteps
	else
		return
	end
	
	// If anything went wrong, default to default footstep at 255
	S_StartSoundAtVolume(player.mo, stepsound or sfx_fstep0, max(stepvolume, minvolume) or 255)
end

// Landing thud effect function
local function landingThud(player)
	local thudsound = nil
	// Maximum landing thud volume is reached when you hit the ground at a speed of thudmax * FRACUNIT
	local thudmax = 10
	// Scale landing thud sound to how fast they hit the ground
	local thudvolume = 255 * min(abs(player.prevmomz or 0), thudmax*FRACUNIT) / thudmax / FRACUNIT
	
	// Player is moving on foot slowly or not at all
	if player.mo.state == S_PLAY_STND or player.mo.state == S_PLAY_WALK
		// Play the slow landing thud
		thudsound = player.feet["landingsfx"]["slow"]
	// Player is, or was, moving on foot quickly
	elseif player.mo.state == S_PLAY_RUN or player.mo.state == S_PLAY_SKID
		// Default to slow landing thud if there isn't a fast thud
		thudsound = player.feet["landingsfx"]["fast"] or player.feet["landingsfx"]["slow"]
	// Player is doing something else so their feet shouldn't be hitting the ground
	else
		return
	end
	
	S_StartSoundAtVolume(player.mo, thudsound or sfx_kc50, thudvolume or 255)
end

addHook("ThinkFrame", function()
	for player in players.iterate()
		// If spectator, get outta here
		if not player.mo
			continue
		end
		
		// Gotta be on the ground for your feet to make noises
		if P_IsObjectOnGround(player.mo) and player.prevground == false
			// Landing thud effects
			landingThud(player)
		else
			// Footstep effects
			footSteps(player)
		end
		
		//Set up vars for next frame's checks
		player.prevframe = player.mo.frame
		player.prevground = P_IsObjectOnGround(player.mo)
		player.prevmomz = player.mo.momz
	end
end, MT_PLAYER)
